from collections.abc import Mapping
from io import BytesIO
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types
from ..types import UNSET, File, FileTypes, Unset

T = TypeVar("T", bound="BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut")


@_attrs_define
class BodyUpdatePromptDatasetProjectsProjectIdPromptDatasetsDatasetIdPut:
    """
    Attributes:
        column_names (Union[None, Unset, list[str]]):
        file (Union[File, None, Unset]):
    """

    column_names: Union[None, Unset, list[str]] = UNSET
    file: Union[File, None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        column_names: Union[None, Unset, list[str]]
        if isinstance(self.column_names, Unset):
            column_names = UNSET
        elif isinstance(self.column_names, list):
            column_names = self.column_names

        else:
            column_names = self.column_names

        file: Union[FileTypes, None, Unset]
        if isinstance(self.file, Unset):
            file = UNSET
        elif isinstance(self.file, File):
            file = self.file.to_tuple()

        else:
            file = self.file

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if column_names is not UNSET:
            field_dict["column_names"] = column_names
        if file is not UNSET:
            field_dict["file"] = file

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        if not isinstance(self.column_names, Unset):
            if isinstance(self.column_names, list):
                for column_names_type_0_item_element in self.column_names:
                    files.append(("column_names", (None, str(column_names_type_0_item_element).encode(), "text/plain")))
            else:
                files.append(("column_names", (None, str(self.column_names).encode(), "text/plain")))

        if not isinstance(self.file, Unset):
            if isinstance(self.file, File):
                files.append(("file", self.file.to_tuple()))
            else:
                files.append(("file", (None, str(self.file).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_column_names(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                return cast(list[str], data)

            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        column_names = _parse_column_names(d.pop("column_names", UNSET))

        def _parse_file(data: object) -> Union[File, None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, bytes):
                    raise TypeError()
                return File(payload=BytesIO(data))

            except:  # noqa: E722
                pass
            return cast(Union[File, None, Unset], data)

        file = _parse_file(d.pop("file", UNSET))

        body_update_prompt_dataset_projects_project_id_prompt_datasets_dataset_id_put = cls(
            column_names=column_names, file=file
        )

        body_update_prompt_dataset_projects_project_id_prompt_datasets_dataset_id_put.additional_properties = d
        return body_update_prompt_dataset_projects_project_id_prompt_datasets_dataset_id_put

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DatasetFilterRows")


@_attrs_define
class DatasetFilterRows:
    """This global operation filters a set of rows and discard the rest.

    Attributes:
        row_ids (list[str]):
        edit_type (Union[Literal['filter_rows'], Unset]):  Default: 'filter_rows'.
    """

    row_ids: list[str]
    edit_type: Union[Literal["filter_rows"], Unset] = "filter_rows"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        row_ids = self.row_ids

        edit_type = self.edit_type

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"row_ids": row_ids})
        if edit_type is not UNSET:
            field_dict["edit_type"] = edit_type

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        row_ids = cast(list[str], d.pop("row_ids"))

        edit_type = cast(Union[Literal["filter_rows"], Unset], d.pop("edit_type", UNSET))
        if edit_type != "filter_rows" and not isinstance(edit_type, Unset):
            raise ValueError(f"edit_type must match const 'filter_rows', got '{edit_type}'")

        dataset_filter_rows = cls(row_ids=row_ids, edit_type=edit_type)

        dataset_filter_rows.additional_properties = d
        return dataset_filter_rows

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.subscription_config import SubscriptionConfig


T = TypeVar("T", bound="PassthroughAction")


@_attrs_define
class PassthroughAction:
    """
    Attributes:
        subscriptions (Union[Unset, list['SubscriptionConfig']]): List of subscriptions to send a notification to when
            this action is applied and the ruleset status matches any of the configured statuses.
        type_ (Union[Literal['PASSTHROUGH'], Unset]):  Default: 'PASSTHROUGH'.
    """

    subscriptions: Union[Unset, list["SubscriptionConfig"]] = UNSET
    type_: Union[Literal["PASSTHROUGH"], Unset] = "PASSTHROUGH"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        subscriptions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.subscriptions, Unset):
            subscriptions = []
            for subscriptions_item_data in self.subscriptions:
                subscriptions_item = subscriptions_item_data.to_dict()
                subscriptions.append(subscriptions_item)

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if subscriptions is not UNSET:
            field_dict["subscriptions"] = subscriptions
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.subscription_config import SubscriptionConfig

        d = dict(src_dict)
        subscriptions = []
        _subscriptions = d.pop("subscriptions", UNSET)
        for subscriptions_item_data in _subscriptions or []:
            subscriptions_item = SubscriptionConfig.from_dict(subscriptions_item_data)

            subscriptions.append(subscriptions_item)

        type_ = cast(Union[Literal["PASSTHROUGH"], Unset], d.pop("type", UNSET))
        if type_ != "PASSTHROUGH" and not isinstance(type_, Unset):
            raise ValueError(f"type must match const 'PASSTHROUGH', got '{type_}'")

        passthrough_action = cls(subscriptions=subscriptions, type_=type_)

        passthrough_action.additional_properties = d
        return passthrough_action

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.column_mapping_config import ColumnMappingConfig


T = TypeVar("T", bound="ColumnMapping")


@_attrs_define
class ColumnMapping:
    """
    Attributes:
        input_ (Union['ColumnMappingConfig', None, list[str]]):
        metadata (Union['ColumnMappingConfig', None, list[str]]):
        output (Union['ColumnMappingConfig', None, list[str]]):
    """

    input_: Union["ColumnMappingConfig", None, list[str]]
    metadata: Union["ColumnMappingConfig", None, list[str]]
    output: Union["ColumnMappingConfig", None, list[str]]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.column_mapping_config import ColumnMappingConfig

        input_: Union[None, dict[str, Any], list[str]]
        if isinstance(self.input_, ColumnMappingConfig):
            input_ = self.input_.to_dict()
        elif isinstance(self.input_, list):
            input_ = self.input_

        else:
            input_ = self.input_

        metadata: Union[None, dict[str, Any], list[str]]
        if isinstance(self.metadata, ColumnMappingConfig):
            metadata = self.metadata.to_dict()
        elif isinstance(self.metadata, list):
            metadata = self.metadata

        else:
            metadata = self.metadata

        output: Union[None, dict[str, Any], list[str]]
        if isinstance(self.output, ColumnMappingConfig):
            output = self.output.to_dict()
        elif isinstance(self.output, list):
            output = self.output

        else:
            output = self.output

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"input": input_, "metadata": metadata, "output": output})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.column_mapping_config import ColumnMappingConfig

        d = dict(src_dict)

        def _parse_input_(data: object) -> Union["ColumnMappingConfig", None, list[str]]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ColumnMappingConfig.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                return cast(list[str], data)

            except:  # noqa: E722
                pass
            return cast(Union["ColumnMappingConfig", None, list[str]], data)

        input_ = _parse_input_(d.pop("input"))

        def _parse_metadata(data: object) -> Union["ColumnMappingConfig", None, list[str]]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ColumnMappingConfig.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                return cast(list[str], data)

            except:  # noqa: E722
                pass
            return cast(Union["ColumnMappingConfig", None, list[str]], data)

        metadata = _parse_metadata(d.pop("metadata"))

        def _parse_output(data: object) -> Union["ColumnMappingConfig", None, list[str]]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ColumnMappingConfig.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                return cast(list[str], data)

            except:  # noqa: E722
                pass
            return cast(Union["ColumnMappingConfig", None, list[str]], data)

        output = _parse_output(d.pop("output"))

        column_mapping = cls(input_=input_, metadata=metadata, output=output)

        column_mapping.additional_properties = d
        return column_mapping

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.and_node import AndNode
    from ..models.filter_leaf import FilterLeaf
    from ..models.log_records_boolean_filter import LogRecordsBooleanFilter
    from ..models.log_records_date_filter import LogRecordsDateFilter
    from ..models.log_records_id_filter import LogRecordsIDFilter
    from ..models.log_records_number_filter import LogRecordsNumberFilter
    from ..models.log_records_sort_clause import LogRecordsSortClause
    from ..models.log_records_text_filter import LogRecordsTextFilter
    from ..models.not_node import NotNode
    from ..models.or_node import OrNode


T = TypeVar("T", bound="RecomputeLogRecordsMetricsRequest")


@_attrs_define
class RecomputeLogRecordsMetricsRequest:
    """Request to recompute metrics for a genai project run (log stream or experiment).
    This request is used to trigger recomputation of metrics based on the provided filters and scorer IDs.

        Attributes:
            scorer_ids (list[str]): List of scorer IDs for which metrics should be recomputed.
            experiment_id (Union[None, Unset, str]): Experiment id associated with the traces.
            filter_tree (Union['AndNode', 'FilterLeaf', 'NotNode', 'OrNode', None, Unset]):
            filters (Union[Unset, list[Union['LogRecordsBooleanFilter', 'LogRecordsDateFilter', 'LogRecordsIDFilter',
                'LogRecordsNumberFilter', 'LogRecordsTextFilter']]]):
            limit (Union[Unset, int]):  Default: 100.
            log_stream_id (Union[None, Unset, str]): Log stream id associated with the traces.
            metrics_testing_id (Union[None, Unset, str]): Metrics testing id associated with the traces.
            sort (Union[Unset, LogRecordsSortClause]):
            starting_token (Union[Unset, int]):  Default: 0.
            truncate_fields (Union[Unset, bool]):  Default: False.
    """

    scorer_ids: list[str]
    experiment_id: Union[None, Unset, str] = UNSET
    filter_tree: Union["AndNode", "FilterLeaf", "NotNode", "OrNode", None, Unset] = UNSET
    filters: Union[
        Unset,
        list[
            Union[
                "LogRecordsBooleanFilter",
                "LogRecordsDateFilter",
                "LogRecordsIDFilter",
                "LogRecordsNumberFilter",
                "LogRecordsTextFilter",
            ]
        ],
    ] = UNSET
    limit: Union[Unset, int] = 100
    log_stream_id: Union[None, Unset, str] = UNSET
    metrics_testing_id: Union[None, Unset, str] = UNSET
    sort: Union[Unset, "LogRecordsSortClause"] = UNSET
    starting_token: Union[Unset, int] = 0
    truncate_fields: Union[Unset, bool] = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.and_node import AndNode
        from ..models.filter_leaf import FilterLeaf
        from ..models.log_records_boolean_filter import LogRecordsBooleanFilter
        from ..models.log_records_date_filter import LogRecordsDateFilter
        from ..models.log_records_id_filter import LogRecordsIDFilter
        from ..models.log_records_number_filter import LogRecordsNumberFilter
        from ..models.not_node import NotNode
        from ..models.or_node import OrNode

        scorer_ids = self.scorer_ids

        experiment_id: Union[None, Unset, str]
        experiment_id = UNSET if isinstance(self.experiment_id, Unset) else self.experiment_id

        filter_tree: Union[None, Unset, dict[str, Any]]
        if isinstance(self.filter_tree, Unset):
            filter_tree = UNSET
        elif isinstance(self.filter_tree, (FilterLeaf, AndNode, OrNode, NotNode)):
            filter_tree = self.filter_tree.to_dict()
        else:
            filter_tree = self.filter_tree

        filters: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = []
            for filters_item_data in self.filters:
                filters_item: dict[str, Any]
                if isinstance(
                    filters_item_data,
                    (LogRecordsIDFilter, LogRecordsDateFilter, LogRecordsNumberFilter, LogRecordsBooleanFilter),
                ):
                    filters_item = filters_item_data.to_dict()
                else:
                    filters_item = filters_item_data.to_dict()

                filters.append(filters_item)

        limit = self.limit

        log_stream_id: Union[None, Unset, str]
        log_stream_id = UNSET if isinstance(self.log_stream_id, Unset) else self.log_stream_id

        metrics_testing_id: Union[None, Unset, str]
        metrics_testing_id = UNSET if isinstance(self.metrics_testing_id, Unset) else self.metrics_testing_id

        sort: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.sort, Unset):
            sort = self.sort.to_dict()

        starting_token = self.starting_token

        truncate_fields = self.truncate_fields

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"scorer_ids": scorer_ids})
        if experiment_id is not UNSET:
            field_dict["experiment_id"] = experiment_id
        if filter_tree is not UNSET:
            field_dict["filter_tree"] = filter_tree
        if filters is not UNSET:
            field_dict["filters"] = filters
        if limit is not UNSET:
            field_dict["limit"] = limit
        if log_stream_id is not UNSET:
            field_dict["log_stream_id"] = log_stream_id
        if metrics_testing_id is not UNSET:
            field_dict["metrics_testing_id"] = metrics_testing_id
        if sort is not UNSET:
            field_dict["sort"] = sort
        if starting_token is not UNSET:
            field_dict["starting_token"] = starting_token
        if truncate_fields is not UNSET:
            field_dict["truncate_fields"] = truncate_fields

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.and_node import AndNode
        from ..models.filter_leaf import FilterLeaf
        from ..models.log_records_boolean_filter import LogRecordsBooleanFilter
        from ..models.log_records_date_filter import LogRecordsDateFilter
        from ..models.log_records_id_filter import LogRecordsIDFilter
        from ..models.log_records_number_filter import LogRecordsNumberFilter
        from ..models.log_records_sort_clause import LogRecordsSortClause
        from ..models.log_records_text_filter import LogRecordsTextFilter
        from ..models.not_node import NotNode
        from ..models.or_node import OrNode

        d = dict(src_dict)
        scorer_ids = cast(list[str], d.pop("scorer_ids"))

        def _parse_experiment_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        experiment_id = _parse_experiment_id(d.pop("experiment_id", UNSET))

        def _parse_filter_tree(data: object) -> Union["AndNode", "FilterLeaf", "NotNode", "OrNode", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return FilterLeaf.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return AndNode.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return OrNode.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return NotNode.from_dict(data)

            except:  # noqa: E722
                pass
            return cast(Union["AndNode", "FilterLeaf", "NotNode", "OrNode", None, Unset], data)

        filter_tree = _parse_filter_tree(d.pop("filter_tree", UNSET))

        filters = []
        _filters = d.pop("filters", UNSET)
        for filters_item_data in _filters or []:

            def _parse_filters_item(
                data: object,
            ) -> Union[
                "LogRecordsBooleanFilter",
                "LogRecordsDateFilter",
                "LogRecordsIDFilter",
                "LogRecordsNumberFilter",
                "LogRecordsTextFilter",
            ]:
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return LogRecordsIDFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return LogRecordsDateFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return LogRecordsNumberFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return LogRecordsBooleanFilter.from_dict(data)

                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                return LogRecordsTextFilter.from_dict(data)

            filters_item = _parse_filters_item(filters_item_data)

            filters.append(filters_item)

        limit = d.pop("limit", UNSET)

        def _parse_log_stream_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        log_stream_id = _parse_log_stream_id(d.pop("log_stream_id", UNSET))

        def _parse_metrics_testing_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        metrics_testing_id = _parse_metrics_testing_id(d.pop("metrics_testing_id", UNSET))

        _sort = d.pop("sort", UNSET)
        sort: Union[Unset, LogRecordsSortClause]
        sort = UNSET if isinstance(_sort, Unset) else LogRecordsSortClause.from_dict(_sort)

        starting_token = d.pop("starting_token", UNSET)

        truncate_fields = d.pop("truncate_fields", UNSET)

        recompute_log_records_metrics_request = cls(
            scorer_ids=scorer_ids,
            experiment_id=experiment_id,
            filter_tree=filter_tree,
            filters=filters,
            limit=limit,
            log_stream_id=log_stream_id,
            metrics_testing_id=metrics_testing_id,
            sort=sort,
            starting_token=starting_token,
            truncate_fields=truncate_fields,
        )

        recompute_log_records_metrics_request.additional_properties = d
        return recompute_log_records_metrics_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from galileo.utils.headers_data import get_package_version
from galileo_core.constants.request_method import RequestMethod
from galileo_core.helpers.api_client import ApiClient

from ... import errors
from ...models.delete_prompt_response import DeletePromptResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import Response


def _get_kwargs(template_id: str) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": RequestMethod.DELETE,
        "return_raw_response": True,
        "path": f"/templates/{template_id}",
    }

    headers["X-Galileo-SDK"] = f"galileo-python/{get_package_version()}"

    _kwargs["content_headers"] = headers
    return _kwargs


def _parse_response(
    *, client: ApiClient, response: httpx.Response
) -> Optional[Union[DeletePromptResponse, HTTPValidationError]]:
    if response.status_code == 200:
        return DeletePromptResponse.from_dict(response.json())

    if response.status_code == 422:
        return HTTPValidationError.from_dict(response.json())

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    return None


def _build_response(
    *, client: ApiClient, response: httpx.Response
) -> Response[Union[DeletePromptResponse, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(template_id: str, *, client: ApiClient) -> Response[Union[DeletePromptResponse, HTTPValidationError]]:
    """Delete Global Template

     Delete a global prompt template given a template ID.

    Parameters
    ----------
    template_id : UUID4
        Prompt template id.
    ctx : Context
        Request context including authentication information

    Returns
    -------
    DeletePromptResponse
        Message indicating the prompt template was deleted.

    Args:
        template_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[DeletePromptResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(template_id=template_id)

    response = client.request(**kwargs)

    return _build_response(client=client, response=response)


def sync(template_id: str, *, client: ApiClient) -> Optional[Union[DeletePromptResponse, HTTPValidationError]]:
    """Delete Global Template

     Delete a global prompt template given a template ID.

    Parameters
    ----------
    template_id : UUID4
        Prompt template id.
    ctx : Context
        Request context including authentication information

    Returns
    -------
    DeletePromptResponse
        Message indicating the prompt template was deleted.

    Args:
        template_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[DeletePromptResponse, HTTPValidationError]
    """

    return sync_detailed(template_id=template_id, client=client).parsed


async def asyncio_detailed(
    template_id: str, *, client: ApiClient
) -> Response[Union[DeletePromptResponse, HTTPValidationError]]:
    """Delete Global Template

     Delete a global prompt template given a template ID.

    Parameters
    ----------
    template_id : UUID4
        Prompt template id.
    ctx : Context
        Request context including authentication information

    Returns
    -------
    DeletePromptResponse
        Message indicating the prompt template was deleted.

    Args:
        template_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[DeletePromptResponse, HTTPValidationError]]
    """

    kwargs = _get_kwargs(template_id=template_id)

    response = await client.arequest(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(template_id: str, *, client: ApiClient) -> Optional[Union[DeletePromptResponse, HTTPValidationError]]:
    """Delete Global Template

     Delete a global prompt template given a template ID.

    Parameters
    ----------
    template_id : UUID4
        Prompt template id.
    ctx : Context
        Request context including authentication information

    Returns
    -------
    DeletePromptResponse
        Message indicating the prompt template was deleted.

    Args:
        template_id (str):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[DeletePromptResponse, HTTPValidationError]
    """

    return (await asyncio_detailed(template_id=template_id, client=client)).parsed



from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ExperimentPrompt")


@_attrs_define
class ExperimentPrompt:
    """
    Attributes:
        content (Union[None, Unset, str]):
        name (Union[None, Unset, str]):
        prompt_template_id (Union[None, Unset, str]):
        version_index (Union[None, Unset, int]):
    """

    content: Union[None, Unset, str] = UNSET
    name: Union[None, Unset, str] = UNSET
    prompt_template_id: Union[None, Unset, str] = UNSET
    version_index: Union[None, Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        content: Union[None, Unset, str]
        content = UNSET if isinstance(self.content, Unset) else self.content

        name: Union[None, Unset, str]
        name = UNSET if isinstance(self.name, Unset) else self.name

        prompt_template_id: Union[None, Unset, str]
        prompt_template_id = UNSET if isinstance(self.prompt_template_id, Unset) else self.prompt_template_id

        version_index: Union[None, Unset, int]
        version_index = UNSET if isinstance(self.version_index, Unset) else self.version_index

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if content is not UNSET:
            field_dict["content"] = content
        if name is not UNSET:
            field_dict["name"] = name
        if prompt_template_id is not UNSET:
            field_dict["prompt_template_id"] = prompt_template_id
        if version_index is not UNSET:
            field_dict["version_index"] = version_index

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_content(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        content = _parse_content(d.pop("content", UNSET))

        def _parse_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_prompt_template_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        prompt_template_id = _parse_prompt_template_id(d.pop("prompt_template_id", UNSET))

        def _parse_version_index(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        version_index = _parse_version_index(d.pop("version_index", UNSET))

        experiment_prompt = cls(
            content=content, name=name, prompt_template_id=prompt_template_id, version_index=version_index
        )

        experiment_prompt.additional_properties = d
        return experiment_prompt

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.boolean_filter import BooleanFilter
    from ..models.collection_filter import CollectionFilter
    from ..models.custom_boolean_filter import CustomBooleanFilter
    from ..models.custom_function_filter import CustomFunctionFilter
    from ..models.custom_number_filter import CustomNumberFilter
    from ..models.custom_uuid_filter import CustomUUIDFilter
    from ..models.date_filter import DateFilter
    from ..models.enum_filter import EnumFilter
    from ..models.id_filter import IDFilter
    from ..models.map_filter import MapFilter
    from ..models.string_filter import StringFilter


T = TypeVar("T", bound="FilterLeaf")


@_attrs_define
class FilterLeaf:
    """
    Attributes:
        filter_ (Union['BooleanFilter', 'CollectionFilter', 'CustomBooleanFilter', 'CustomFunctionFilter',
            'CustomNumberFilter', 'CustomUUIDFilter', 'DateFilter', 'EnumFilter', 'IDFilter', 'MapFilter', 'StringFilter']):
    """

    filter_: Union[
        "BooleanFilter",
        "CollectionFilter",
        "CustomBooleanFilter",
        "CustomFunctionFilter",
        "CustomNumberFilter",
        "CustomUUIDFilter",
        "DateFilter",
        "EnumFilter",
        "IDFilter",
        "MapFilter",
        "StringFilter",
    ]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.boolean_filter import BooleanFilter
        from ..models.collection_filter import CollectionFilter
        from ..models.custom_boolean_filter import CustomBooleanFilter
        from ..models.custom_number_filter import CustomNumberFilter
        from ..models.custom_uuid_filter import CustomUUIDFilter
        from ..models.date_filter import DateFilter
        from ..models.enum_filter import EnumFilter
        from ..models.id_filter import IDFilter
        from ..models.map_filter import MapFilter
        from ..models.string_filter import StringFilter

        filter_: dict[str, Any]
        if isinstance(
            self.filter_,
            (
                IDFilter,
                CustomUUIDFilter,
                DateFilter,
                BooleanFilter,
                CustomNumberFilter,
                EnumFilter,
                MapFilter,
                CollectionFilter,
                StringFilter,
                CustomBooleanFilter,
            ),
        ):
            filter_ = self.filter_.to_dict()
        else:
            filter_ = self.filter_.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"filter": filter_})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.boolean_filter import BooleanFilter
        from ..models.collection_filter import CollectionFilter
        from ..models.custom_boolean_filter import CustomBooleanFilter
        from ..models.custom_function_filter import CustomFunctionFilter
        from ..models.custom_number_filter import CustomNumberFilter
        from ..models.custom_uuid_filter import CustomUUIDFilter
        from ..models.date_filter import DateFilter
        from ..models.enum_filter import EnumFilter
        from ..models.id_filter import IDFilter
        from ..models.map_filter import MapFilter
        from ..models.string_filter import StringFilter

        d = dict(src_dict)

        def _parse_filter_(
            data: object,
        ) -> Union[
            "BooleanFilter",
            "CollectionFilter",
            "CustomBooleanFilter",
            "CustomFunctionFilter",
            "CustomNumberFilter",
            "CustomUUIDFilter",
            "DateFilter",
            "EnumFilter",
            "IDFilter",
            "MapFilter",
            "StringFilter",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return IDFilter.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return CustomUUIDFilter.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return DateFilter.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return BooleanFilter.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return CustomNumberFilter.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return EnumFilter.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return MapFilter.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return CollectionFilter.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return StringFilter.from_dict(data)

            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return CustomBooleanFilter.from_dict(data)

            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            return CustomFunctionFilter.from_dict(data)

        filter_ = _parse_filter_(d.pop("filter"))

        filter_leaf = cls(filter_=filter_)

        filter_leaf.additional_properties = d
        return filter_leaf

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import logging
from typing import Optional

from galileo.config import GalileoPythonConfig
from galileo.resources.api.jobs import create_job_jobs_post
from galileo.resources.models import CreateJobRequest, CreateJobResponse, PromptRunSettings, ScorerConfig, TaskType
from galileo_core.exceptions.http import GalileoHTTPException

_logger = logging.getLogger(__name__)


class Jobs:
    config: GalileoPythonConfig

    def __init__(self) -> None:
        self.config = GalileoPythonConfig.get()

    def create(
        self,
        project_id: str,
        name: str,
        run_id: str,
        dataset_id: str,
        prompt_template_id: str,
        task_type: TaskType,
        scorers: Optional[list[ScorerConfig]],
        prompt_settings: PromptRunSettings,
    ) -> CreateJobResponse:
        create_params = {
            "project_id": project_id,
            "dataset_id": dataset_id,
            "job_name": name,
            "run_id": run_id,
            "prompt_settings": prompt_settings,
            "prompt_template_version_id": prompt_template_id,
            "task_type": task_type,
            "scorers": scorers,
        }
        _logger.info(f"create job: {create_params}")
        result = create_job_jobs_post.sync_detailed(
            client=self.config.api_client, body=CreateJobRequest(**create_params)
        )
        if not result.parsed or not isinstance(result.parsed, CreateJobResponse):
            raise GalileoHTTPException(
                message="Create job failed", status_code=result.status_code, response_text=str(result.content)
            )
        return result.parsed



from http import HTTPStatus
from typing import Any, Optional

import httpx

from galileo.utils.headers_data import get_package_version
from galileo_core.constants.request_method import RequestMethod
from galileo_core.helpers.api_client import ApiClient

from ... import errors
from ...models.healthcheck_response import HealthcheckResponse
from ...types import Response


def _get_kwargs() -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {"method": RequestMethod.GET, "return_raw_response": True, "path": "/healthcheck"}

    headers["X-Galileo-SDK"] = f"galileo-python/{get_package_version()}"

    _kwargs["content_headers"] = headers
    return _kwargs


def _parse_response(*, client: ApiClient, response: httpx.Response) -> Optional[HealthcheckResponse]:
    if response.status_code == 200:
        return HealthcheckResponse.from_dict(response.json())

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    return None


def _build_response(*, client: ApiClient, response: httpx.Response) -> Response[HealthcheckResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(*, client: ApiClient) -> Response[HealthcheckResponse]:
    """Healthcheck

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HealthcheckResponse]
    """

    kwargs = _get_kwargs()

    response = client.request(**kwargs)

    return _build_response(client=client, response=response)


def sync(*, client: ApiClient) -> Optional[HealthcheckResponse]:
    """Healthcheck

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HealthcheckResponse
    """

    return sync_detailed(client=client).parsed


async def asyncio_detailed(*, client: ApiClient) -> Response[HealthcheckResponse]:
    """Healthcheck

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HealthcheckResponse]
    """

    kwargs = _get_kwargs()

    response = await client.arequest(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(*, client: ApiClient) -> Optional[HealthcheckResponse]:
    """Healthcheck

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HealthcheckResponse
    """

    return (await asyncio_detailed(client=client)).parsed



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.log_records_metrics_response_aggregate_metrics import LogRecordsMetricsResponseAggregateMetrics
    from ..models.log_records_metrics_response_bucketed_metrics import LogRecordsMetricsResponseBucketedMetrics


T = TypeVar("T", bound="LogRecordsMetricsResponse")


@_attrs_define
class LogRecordsMetricsResponse:
    """
    Attributes:
        aggregate_metrics (LogRecordsMetricsResponseAggregateMetrics):
        bucketed_metrics (LogRecordsMetricsResponseBucketedMetrics):
        group_by_columns (list[str]):
    """

    aggregate_metrics: "LogRecordsMetricsResponseAggregateMetrics"
    bucketed_metrics: "LogRecordsMetricsResponseBucketedMetrics"
    group_by_columns: list[str]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        aggregate_metrics = self.aggregate_metrics.to_dict()

        bucketed_metrics = self.bucketed_metrics.to_dict()

        group_by_columns = self.group_by_columns

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "aggregate_metrics": aggregate_metrics,
                "bucketed_metrics": bucketed_metrics,
                "group_by_columns": group_by_columns,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.log_records_metrics_response_aggregate_metrics import LogRecordsMetricsResponseAggregateMetrics
        from ..models.log_records_metrics_response_bucketed_metrics import LogRecordsMetricsResponseBucketedMetrics

        d = dict(src_dict)
        aggregate_metrics = LogRecordsMetricsResponseAggregateMetrics.from_dict(d.pop("aggregate_metrics"))

        bucketed_metrics = LogRecordsMetricsResponseBucketedMetrics.from_dict(d.pop("bucketed_metrics"))

        group_by_columns = cast(list[str], d.pop("group_by_columns"))

        log_records_metrics_response = cls(
            aggregate_metrics=aggregate_metrics, bucketed_metrics=bucketed_metrics, group_by_columns=group_by_columns
        )

        log_records_metrics_response.additional_properties = d
        return log_records_metrics_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.stage_type import StageType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ruleset import Ruleset


T = TypeVar("T", bound="StageWithRulesets")


@_attrs_define
class StageWithRulesets:
    """
    Attributes:
        name (str): Name of the stage. Must be unique within the project.
        project_id (str): ID of the project to which this stage belongs.
        description (Union[None, Unset, str]): Optional human-readable description of the goals of this guardrail.
        paused (Union[Unset, bool]): Whether the action is enabled. If False, the action will not be applied. Default:
            False.
        prioritized_rulesets (Union[Unset, list['Ruleset']]): Rulesets to be applied to the payload.
        type_ (Union[Unset, StageType]):
    """

    name: str
    project_id: str
    description: Union[None, Unset, str] = UNSET
    paused: Union[Unset, bool] = False
    prioritized_rulesets: Union[Unset, list["Ruleset"]] = UNSET
    type_: Union[Unset, StageType] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        project_id = self.project_id

        description: Union[None, Unset, str]
        description = UNSET if isinstance(self.description, Unset) else self.description

        paused = self.paused

        prioritized_rulesets: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.prioritized_rulesets, Unset):
            prioritized_rulesets = []
            for prioritized_rulesets_item_data in self.prioritized_rulesets:
                prioritized_rulesets_item = prioritized_rulesets_item_data.to_dict()
                prioritized_rulesets.append(prioritized_rulesets_item)

        type_: Union[Unset, str] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"name": name, "project_id": project_id})
        if description is not UNSET:
            field_dict["description"] = description
        if paused is not UNSET:
            field_dict["paused"] = paused
        if prioritized_rulesets is not UNSET:
            field_dict["prioritized_rulesets"] = prioritized_rulesets
        if type_ is not UNSET:
            field_dict["type"] = type_

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ruleset import Ruleset

        d = dict(src_dict)
        name = d.pop("name")

        project_id = d.pop("project_id")

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        paused = d.pop("paused", UNSET)

        prioritized_rulesets = []
        _prioritized_rulesets = d.pop("prioritized_rulesets", UNSET)
        for prioritized_rulesets_item_data in _prioritized_rulesets or []:
            prioritized_rulesets_item = Ruleset.from_dict(prioritized_rulesets_item_data)

            prioritized_rulesets.append(prioritized_rulesets_item)

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, StageType]
        type_ = UNSET if isinstance(_type_, Unset) else StageType(_type_)

        stage_with_rulesets = cls(
            name=name,
            project_id=project_id,
            description=description,
            paused=paused,
            prioritized_rulesets=prioritized_rulesets,
            type_=type_,
        )

        stage_with_rulesets.additional_properties = d
        return stage_with_rulesets

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Sequence
from typing import Optional

from langchain_core.runnables.base import Runnable
from langchain_core.tools import BaseTool
from pydantic import UUID4, BaseModel, ConfigDict, Field
from pydantic.v1 import BaseModel as BaseModelV1

from galileo.constants.protect import TIMEOUT_SECS
from galileo.protect import ainvoke_protect, invoke_protect
from galileo.utils.logging import get_logger
from galileo_core.schemas.protect.execution_status import ExecutionStatus
from galileo_core.schemas.protect.payload import Payload as CorePayload
from galileo_core.schemas.protect.response import Response
from galileo_core.schemas.protect.ruleset import Ruleset

logger = get_logger(__name__)


class ProtectToolInputSchema(BaseModelV1):
    input: Optional[str] = None
    output: Optional[str] = None


class ProtectTool(BaseTool):
    """A LangChain tool that wraps the Galileo Protect API.

    This tool allows you to integrate Galileo Protect into your LangChain applications
    to scan for and mitigate harmful content in both inputs and outputs. It can be
    configured with specific rulesets and linked to a Galileo project and stage for
    monitoring and management.

    It accepts an input and/or output as arguments (parsed by a Pydantic model) and returns a JSON-serialized ``Response`` object from the Protect API.

    Attributes:
        prioritized_rulesets: An optional sequence of ``Ruleset`` objects to apply.
        project_id: The UUID of the Galileo project this tool is associated with.
        project_name: The name of the Galileo project.
        stage_name: The name of the Protect stage to use for this tool.
        stage_id: The UUID of the Protect stage.
        stage_version: The version of the Protect stage to use.
        timeout: The timeout in seconds for the API request.
    """

    name: str = "GalileoProtect"
    description: str = (
        "Protect your LLM applications from harmful content using Galileo Protect. "
        "This tool is a wrapper around Galileo's Protect API, can be used to scan text "
        "for harmful content, and can be used to trigger actions based on the results."
        "The tool can be used on the input text or output text, and can be configured "
        "with a set of rulesets to evaluate on."
    )
    # langchain requires pydantic v1
    args_schema: type[BaseModelV1] = ProtectToolInputSchema

    prioritized_rulesets: Optional[Sequence[Ruleset]] = None
    project_id: Optional[UUID4] = None
    project_name: Optional[str] = None
    stage_name: Optional[str] = None
    stage_id: Optional[UUID4] = None
    stage_version: Optional[int] = None
    timeout: float = TIMEOUT_SECS

    def _run(self, input: Optional[str] = None, output: Optional[str] = None) -> str:
        """
        Apply the tool synchronously.

        We serialize the response to JSON because that's what `langchain_core` expects
        for tools.
        """
        api_payload = CorePayload(input=input, output=output)
        api_response = invoke_protect(
            payload=api_payload,
            prioritized_rulesets=self.prioritized_rulesets,
            project_name=self.project_name,
            project_id=self.project_id,
            stage_name=self.stage_name,
            stage_id=self.stage_id,
            stage_version=self.stage_version,
            timeout=self.timeout,
        )
        return api_response.model_dump_json()

    async def _arun(self, input: Optional[str] = None, output: Optional[str] = None) -> str:
        """
        Apply the tool asynchronously.

        We serialize the response to JSON because that's what `langchain_core` expects
        for tools.
        """
        api_payload = CorePayload(input=input, output=output)
        api_response = await ainvoke_protect(
            payload=api_payload,
            prioritized_rulesets=self.prioritized_rulesets,
            project_name=self.project_name,
            project_id=self.project_id,
            stage_name=self.stage_name,
            stage_id=self.stage_id,
            stage_version=self.stage_version,
            timeout=self.timeout,
        )
        return api_response.model_dump_json()


class ProtectParser(BaseModel):
    """A LangChain runnable that parses and routes the output of a ``ProtectTool``.

    If the Protect API response is 'triggered', it returns the response text.
    Otherwise, it invokes a fallback chain.

    Attributes:
        chain: The ``Runnable`` to invoke if the Protect invocation is not triggered.
        ignore_trigger: If True, always invoke the fallback chain.
        echo_output: If True, print the raw Protect API response to the console.
    """

    chain: Runnable = Field(..., description="The chain to trigger if the Protect invocation is not triggered.")
    ignore_trigger: bool = Field(
        default=False,
        description="Ignore the status of the Protect invocation and always trigger the rest of the chain.",
    )
    echo_output: bool = Field(default=False, description="Echo the output of the Protect invocation.")

    model_config = ConfigDict(arbitrary_types_allowed=True)

    def parser(self, response_raw_json: str) -> str:
        """Parses the JSON response from ``ProtectTool`` and decides the execution path.

        If the response status is 'triggered', the response text is returned. Otherwise,
        the fallback chain is invoked with the text.

        If JSON parsing fails, it assumes the input is not from ``ProtectTool`` and
        invokes the fallback chain directly with the raw input.

        Args:
            response_raw_json: Expects the output from the ``ProtectTool``.

        Returns:
            The text from the Protect response if triggered, or the result of invoking
            the fallback chain.
        """
        try:
            response = Response.model_validate_json(response_raw_json)
        except Exception:
            return self.chain.invoke(response_raw_json)
        text = response.text

        if self.echo_output:
            logger.debug(f"> Raw response: {text}")
        if response.status == ExecutionStatus.triggered and not self.ignore_trigger:
            return text
        return self.chain.invoke(text)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.aggregated_trace_view_edge import AggregatedTraceViewEdge
    from ..models.aggregated_trace_view_node import AggregatedTraceViewNode


T = TypeVar("T", bound="AggregatedTraceViewGraph")


@_attrs_define
class AggregatedTraceViewGraph:
    """
    Attributes:
        edges (list['AggregatedTraceViewEdge']):
        nodes (list['AggregatedTraceViewNode']):
    """

    edges: list["AggregatedTraceViewEdge"]
    nodes: list["AggregatedTraceViewNode"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        edges = []
        for edges_item_data in self.edges:
            edges_item = edges_item_data.to_dict()
            edges.append(edges_item)

        nodes = []
        for nodes_item_data in self.nodes:
            nodes_item = nodes_item_data.to_dict()
            nodes.append(nodes_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"edges": edges, "nodes": nodes})

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.aggregated_trace_view_edge import AggregatedTraceViewEdge
        from ..models.aggregated_trace_view_node import AggregatedTraceViewNode

        d = dict(src_dict)
        edges = []
        _edges = d.pop("edges")
        for edges_item_data in _edges:
            edges_item = AggregatedTraceViewEdge.from_dict(edges_item_data)

            edges.append(edges_item)

        nodes = []
        _nodes = d.pop("nodes")
        for nodes_item_data in _nodes:
            nodes_item = AggregatedTraceViewNode.from_dict(nodes_item_data)

            nodes.append(nodes_item)

        aggregated_trace_view_graph = cls(edges=edges, nodes=nodes)

        aggregated_trace_view_graph.additional_properties = d
        return aggregated_trace_view_graph

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import builtins
import mimetypes
import time
from typing import Any, Optional, Union, overload

from galileo.config import GalileoPythonConfig
from galileo.resources.api.datasets import (
    create_dataset_datasets_post,
    delete_dataset_datasets_dataset_id_delete,
    extend_dataset_content_datasets_extend_post,
    get_dataset_content_datasets_dataset_id_content_get,
    get_dataset_datasets_dataset_id_get,
    get_dataset_synthetic_extend_status_datasets_extend_dataset_id_get,
    get_dataset_version_content_datasets_dataset_id_versions_version_index_content_get,
    list_datasets_datasets_get,
    query_dataset_versions_datasets_dataset_id_versions_query_post,
    query_datasets_datasets_query_post,
    update_dataset_content_datasets_dataset_id_content_patch,
)
from galileo.resources.models import DatasetRow, ListDatasetVersionParams, ListDatasetVersionResponse
from galileo.resources.models.body_create_dataset_datasets_post import BodyCreateDatasetDatasetsPost
from galileo.resources.models.dataset_append_row import DatasetAppendRow
from galileo.resources.models.dataset_append_row_values import DatasetAppendRowValues
from galileo.resources.models.dataset_content import DatasetContent
from galileo.resources.models.dataset_db import DatasetDB
from galileo.resources.models.dataset_name_filter import DatasetNameFilter
from galileo.resources.models.dataset_name_filter_operator import DatasetNameFilterOperator
from galileo.resources.models.dataset_updated_at_sort import DatasetUpdatedAtSort
from galileo.resources.models.http_validation_error import HTTPValidationError
from galileo.resources.models.job_progress import JobProgress
from galileo.resources.models.list_dataset_params import ListDatasetParams
from galileo.resources.models.list_dataset_response import ListDatasetResponse
from galileo.resources.models.prompt_run_settings import PromptRunSettings
from galileo.resources.models.synthetic_data_types import SyntheticDataTypes
from galileo.resources.models.synthetic_dataset_extension_request import SyntheticDatasetExtensionRequest
from galileo.resources.models.synthetic_dataset_extension_response import SyntheticDatasetExtensionResponse
from galileo.resources.models.update_dataset_content_request import UpdateDatasetContentRequest
from galileo.resources.types import File, Unset
from galileo.schema.datasets import DatasetRecord
from galileo.utils.catch_log import DecorateAllMethods
from galileo.utils.exceptions import APIException
from galileo.utils.logging import get_logger
from galileo_core.utils.dataset import DatasetType, parse_dataset

logger = get_logger(__name__)
MAX_DATASET_ROWS = 100000


class DatasetAPIException(APIException):
    pass


class Dataset(DecorateAllMethods):
    content: Optional[DatasetContent] = None
    config: GalileoPythonConfig

    def __init__(self, dataset_db: DatasetDB) -> None:
        self.dataset = dataset_db
        self.config = GalileoPythonConfig.get()

    def get_content(self) -> Union[None, DatasetContent]:
        """
        Gets and returns the content of the dataset.
        Also refreshes the content of the local dataset instance.

        Returns
        -------
        Union[None, DatasetContent]
            The content of the dataset

        Raises
        ------
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

        """
        if not self.dataset:
            return None

        content: DatasetContent = get_dataset_content_datasets_dataset_id_content_get.sync(
            client=self.config.api_client, dataset_id=self.dataset.id, limit=MAX_DATASET_ROWS
        )

        self.content = content

        return content

    def _get_etag(self) -> Optional[str]:
        """
        ETag is returned in response headers of API endpoints of the format /datasets.*contents.*

        This is a required parameter to be passed along with all dataset update requests to ensure
        there isn't a version conflict during updates.
        """
        if not self.dataset:
            return None

        response = get_dataset_content_datasets_dataset_id_content_get.sync_detailed(
            client=self.config.api_client, dataset_id=self.dataset.id
        )

        return response.headers.get("ETag")

    def add_rows(self, row_data: list[dict[str, Any]]) -> "Dataset":
        """
        Adds rows to the dataset.

        Parameters
        ----------
        row_data : List[Dict[str, Any]]
            The rows to add to the dataset.

        Returns
        -------
        Dataset
            The updated dataset with the new rows.

        Raises
        ------
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

        """
        append_rows: list[DatasetAppendRow] = [
            DatasetAppendRow(values=DatasetAppendRowValues.from_dict(row)) for row in row_data
        ]
        request = UpdateDatasetContentRequest(edits=append_rows)
        response = update_dataset_content_datasets_dataset_id_content_patch.sync(
            client=self.config.api_client, dataset_id=self.dataset.id, body=request, if_match=self._get_etag()
        )
        if isinstance(response, HTTPValidationError):
            raise DatasetAPIException("Request to add new rows to dataset failed.")

        # Refresh the content
        self.get_content()

        return self

    def get_version_history(self) -> Optional[Union[HTTPValidationError, ListDatasetVersionResponse]]:
        return query_dataset_versions_datasets_dataset_id_versions_query_post.sync(
            dataset_id=self.dataset.id, client=self.config.api_client, body=ListDatasetVersionParams()
        )

    def load_version(self, version_index: int) -> DatasetContent:
        return get_dataset_version_content_datasets_dataset_id_versions_version_index_content_get.sync(
            dataset_id=self.dataset.id, version_index=version_index, client=self.config.api_client
        )

    def __getattr__(self, attr: str) -> Any:
        """
        Delegate attribute access to the underlying DatasetDB instance.
        """
        return getattr(self.dataset, attr)


class Datasets:
    config: GalileoPythonConfig

    def __init__(self) -> None:
        self.config = GalileoPythonConfig.get()

    def list(self, limit: Union[Unset, int] = 100) -> list[Dataset]:
        """
        Lists all datasets.

        Parameters
        ----------
        limit : Union[Unset, int]
            The maximum number of datasets to return. Default is 100.

        Returns
        -------
        List[Dataset]
            A list of datasets.

        Raises
        ------
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

        """
        datasets: ListDatasetResponse = list_datasets_datasets_get.sync(client=self.config.api_client, limit=limit)
        return [Dataset(dataset_db=dataset) for dataset in datasets.datasets] if datasets else []

    @overload
    def get(self, *, id: str, with_content: bool = False) -> Optional[Dataset]: ...
    @overload
    @overload
    def get(self, *, name: str, with_content: bool = False) -> Optional[Dataset]: ...
    def get(
        self, *, id: Optional[str] = None, name: Optional[str] = None, with_content: bool = False
    ) -> Optional[Dataset]:
        """
        Retrieves a dataset by id or name (exactly one of `id` or `name` must be provided).

        Parameters
        ----------
        id : str
            The id of the dataset.
        name : str
            The name of the dataset.
        with_content : bool
            Whether to return the content of the dataset. Default is False.

        Returns
        -------
        Dataset
            The dataset.

        Raises
        ------
        ValueError
            If neither or both `id` and `name` are provided.
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

        """
        if (id is None) == (name is None):
            raise ValueError("Exactly one of 'id' or 'name' must be provided")

        if id:
            dataset_response = get_dataset_datasets_dataset_id_get.sync(client=self.config.api_client, dataset_id=id)
            if not dataset_response:
                return None
            dataset = Dataset(dataset_db=dataset_response)

        elif name:
            filter = DatasetNameFilter(operator=DatasetNameFilterOperator.EQ, value=name)
            params = ListDatasetParams(filters=[filter], sort=DatasetUpdatedAtSort(ascending=False))
            datasets_response: ListDatasetResponse = query_datasets_datasets_query_post.sync(
                client=self.config.api_client, body=params, limit=1
            )

            if not datasets_response or len(datasets_response.datasets) == 0:
                return None

            dataset = Dataset(dataset_db=datasets_response.datasets[0])

        if with_content:
            dataset.get_content()
        return dataset

    @overload
    def delete(self, *, id: str) -> None: ...
    @overload
    def delete(self, *, name: str) -> None: ...
    def delete(self, *, id: Optional[str] = None, name: Optional[str] = None) -> None:
        """
        Deletes a dataset by id or name.

        Parameters
        ----------
        id : str
            The id of the dataset.
        name : str
            The name of the dataset.

        Raises
        ------
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

        """
        if (id is None) == (name is None):
            raise ValueError("Exactly one of 'id' or 'name' must be provided")

        dataset = self.get(id=id, name=name)  # type: ignore[call-overload]

        if not dataset:
            raise ValueError(f"Dataset {name} not found")
        return delete_dataset_datasets_dataset_id_delete.sync(client=self.config.api_client, dataset_id=dataset.id)

    def create(self, name: str, content: DatasetType) -> Dataset:
        """
        Creates a new dataset.

        Parameters
        ----------
        name : str
            The name of the dataset.
        content : DatasetType
            The content of the dataset.

        Returns
        -------
        Dataset
            The created dataset.

        Raises
        ------
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

        """
        if isinstance(content, (list, dict)) and len(content) == 0:
            # we want to avoid errors: Invalid CSV data: CSV parse error:
            # Empty CSV file or block: cannot infer number of columns
            content = [{}]
        file_path, dataset_format = parse_dataset(content)
        file = File(
            payload=file_path.open("rb"),
            file_name=name,
            mime_type=mimetypes.guess_type(file_path)[0] or "application/octet-stream",
        )

        body = BodyCreateDatasetDatasetsPost(file=file, name=name)

        detailed_response = create_dataset_datasets_post.sync_detailed(
            client=self.config.api_client, body=body, format_=dataset_format
        )

        if not detailed_response.parsed or isinstance(detailed_response.parsed, HTTPValidationError):
            raise DatasetAPIException(detailed_response.content)

        return Dataset(dataset_db=detailed_response.parsed)

    def extend(
        self,
        *,
        prompt_settings: Optional[dict[str, Any]] = None,
        prompt: Optional[str] = None,
        instructions: Optional[str] = None,
        examples: Optional[builtins.list[str]] = None,
        data_types: Optional[builtins.list[str]] = None,
        count: int = 10,
    ) -> builtins.list[DatasetRow]:
        """
        Extends a dataset with synthetically generated data based on the provided parameters.

        This method initiates a dataset extension job, waits for it to complete by polling its status,
        and then returns the content of the extended dataset.

        Parameters
        ----------
        prompt_settings : Dict[str, Any], optional
            Settings for the prompt generation. Should contain 'model_alias' key.
            Example: {'model_alias': 'GPT-4o mini'}
        prompt : str, optional
            A description of the assistant's role.
        instructions : str, optional
            Instructions for the assistant.
        examples : List[str], optional
            Examples of user prompts.
        data_types : List[str], optional
            The types of data to generate. Possible values are:
            'General Query', 'Prompt Injection', 'Off-Topic Query',
            'Toxic Content in Query', 'Multiple Questions in Query',
            'Sexist Content in Query'.
        count : int, default 10
            The number of synthetic examples to generate.

        Returns
        -------
        List[DatasetRow]
            A list of rows from the extended dataset.

        Raises
        ------
        DatasetAPIException
            If the request to extend the dataset fails.
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.
        """
        # Convert prompt_settings dict to PromptRunSettings if provided
        model_alias = prompt_settings.get("model_alias", "gpt-4.1-mini") if prompt_settings else "gpt-4.1-mini"
        prompt_run_settings = PromptRunSettings(model_alias=model_alias)

        # Convert data_types strings to SyntheticDataTypes enum if provided
        synthetic_data_types = None
        if data_types:
            synthetic_data_types = []
            for data_type in data_types:
                try:
                    synthetic_data_types.append(SyntheticDataTypes(data_type))
                except ValueError:
                    raise ValueError(
                        f"Invalid data_type: {data_type}. Must be one of: {[dt.value for dt in SyntheticDataTypes]}"
                    )

        # Create the extension request
        request = SyntheticDatasetExtensionRequest(
            count=count,
            data_types=synthetic_data_types,
            examples=examples,
            instructions=instructions,
            prompt=prompt,
            prompt_settings=prompt_run_settings,
        )

        # Start the extension job
        response = extend_dataset_content_datasets_extend_post.sync(client=self.config.api_client, body=request)

        if isinstance(response, HTTPValidationError):
            raise DatasetAPIException("Request to extend dataset failed.")

        if not response or not isinstance(response, SyntheticDatasetExtensionResponse):
            raise DatasetAPIException("Invalid response from extend dataset API.")

        dataset_id = response.dataset_id

        # Poll for job completion
        while True:
            job_progress = get_dataset_synthetic_extend_status_datasets_extend_dataset_id_get.sync(
                dataset_id=dataset_id, client=self.config.api_client
            )

            if isinstance(job_progress, HTTPValidationError):
                raise DatasetAPIException("Failed to get dataset extension status.")

            if not job_progress or not isinstance(job_progress, JobProgress):
                raise DatasetAPIException("Invalid job progress response.")

            # Check if job is complete
            if (
                job_progress.steps_completed is not None
                and job_progress.steps_total is not None
                and job_progress.steps_completed == job_progress.steps_total
            ):
                logger.info(
                    f"({job_progress.steps_completed}/{job_progress.steps_total}) {job_progress.progress_message}"
                )
                break

            # Log progress message if available
            if job_progress.progress_message:
                logger.info(
                    f"({job_progress.steps_completed}/{job_progress.steps_total}) {job_progress.progress_message}"
                )

            # Wait 1 second before polling again
            time.sleep(1)

        # Get the final dataset content
        dataset_content = get_dataset_content_datasets_dataset_id_content_get.sync(
            client=self.config.api_client, dataset_id=dataset_id
        )

        if not dataset_content or not dataset_content.rows:
            return []

        return dataset_content.rows


#
# Convenience methods
#


def get_dataset(*, id: Optional[str] = None, name: Optional[str] = None) -> Optional[Dataset]:
    """
    Retrieves a dataset by id or name (exactly one of `id` or `name` must be provided).

    Parameters
    ----------
    id : str
        The id of the dataset.
    name : str
        The name of the dataset.
    with_content : bool
        Whether to return the content of the dataset. Default is False.

    Returns
    -------
    Dataset
        The dataset.

    Raises
    ------
    errors.UnexpectedStatus
        If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
    httpx.TimeoutException
        If the request takes longer than Client.timeout.

    """
    return Datasets().get(id=id, name=name)  # type: ignore[call-overload]


def list_datasets(limit: Union[Unset, int] = 100) -> list[Dataset]:
    """
    Lists all datasets.

    Parameters
    ----------
    limit : Union[Unset, int]
        The maximum number of datasets to return. Default is 100.

    Returns
    -------
    List[Dataset]
        A list of datasets.

    Raises
        ------
        errors.UnexpectedStatus
            If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException
            If the request takes longer than Client.timeout.

    """
    return Datasets().list(limit=limit)


def delete_dataset(*, id: Optional[str] = None, name: Optional[str] = None) -> None:
    """
    Deletes a dataset by id or name (exactly one of `id` or `name` must be provided).

    Parameters
    ----------
    id : str
        The id of the dataset.
    name : str
        The name of the dataset.

    Raises
    ------
    errors.UnexpectedStatus
        If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
    httpx.TimeoutException
        If the request takes longer than Client.timeout.

    """
    return Datasets().delete(id=id, name=name)  # type: ignore[call-overload]


def create_dataset(name: str, content: DatasetType) -> Dataset:
    """
    Creates a new dataset.

    Parameters
    ----------
    name : str
        The name of the dataset.
    content : DatasetType
        The content of the dataset.

    Returns
    -------
    Dataset
        The created dataset.

    Raises
    ------
    errors.UnexpectedStatus
        If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
    httpx.TimeoutException
        If the request takes longer than Client.timeout.

    """
    return Datasets().create(name=name, content=content)


def get_dataset_version_history(
    *, dataset_name: Optional[str] = None, dataset_id: Optional[str] = None
) -> Optional[Union[HTTPValidationError, ListDatasetVersionResponse]]:
    """
    Retrieves a dataset version history by dataset name or dataset id.

    Parameters
    ----------
    dataset_name: str
        The name of the dataset.
    dataset_id: str
        The id of the dataset.

    Returns
    -------
    ListDatasetVersionResponse

    Raises
    ------
    HTTPValidationError
    """
    if dataset_name is not None:
        dataset = Datasets().get(name=dataset_name)
        if dataset is None:
            raise ValueError(f"Dataset '{dataset_name}' not found")
        return dataset.get_version_history()
    if dataset_id is not None:
        dataset = Datasets().get(id=dataset_id)
        if dataset is None:
            raise ValueError(f"Dataset '{dataset_id}' not found")
        return dataset.get_version_history()
    raise ValueError("Either dataset_name or dataset_id must be provided.")


def get_dataset_version(
    *, version_index: int, dataset_name: Optional[str] = None, dataset_id: Optional[str] = None
) -> Optional[DatasetContent]:
    """
    Retrieves a dataset version by dataset name or dataset id.

    Parameters
    ----------
    version_index : int
        The version of the dataset.

    dataset_name: Optional[str]
        The name of the dataset.

    dataset_id: Optional[str]
        The id of the dataset.

    Returns
    -------
    DatasetContent
    """
    if dataset_name is not None:
        dataset = Datasets().get(name=dataset_name)
        if dataset is None:
            raise ValueError(f"Dataset '{dataset_name}' not found")
        return dataset.load_version(version_index)

    if dataset_id is not None:
        dataset = Datasets().get(id=dataset_id)
        if dataset is None:
            raise ValueError(f"Dataset '{dataset_id}' not found")
        return dataset.load_version(version_index)
    raise ValueError("Either dataset_name or dataset_id must be provided.")


def extend_dataset(
    *,
    prompt_settings: Optional[dict[str, Any]] = None,
    prompt: Optional[str] = None,
    instructions: Optional[str] = None,
    examples: Optional[list[str]] = None,
    data_types: Optional[list[str]] = None,
    count: int = 10,
) -> list[DatasetRow]:
    """
    Extends a dataset with synthetically generated data based on the provided parameters.

    This function initiates a dataset extension job, waits for it to complete by polling its status,
    and then returns the content of the extended dataset.

    Parameters
    ----------
    prompt_settings : Dict[str, Any], optional
        Settings for the prompt generation. Should contain 'model_alias' key.
        Example: {'model_alias': 'GPT-4o mini'}
    prompt : str, optional
        A description of the assistant's role.
    instructions : str, optional
        Instructions for the assistant.
    examples : List[str], optional
        Examples of user prompts.
    data_types : List[str], optional
        The types of data to generate. Possible values are:
        'General Query', 'Prompt Injection', 'Off-Topic Query',
        'Toxic Content in Query', 'Multiple Questions in Query',
        'Sexist Content in Query'.
    count : int, default 10
        The number of synthetic examples to generate.

    Returns
    -------
    List[DatasetRow]
        A list of rows from the extended dataset.

    Raises
    ------
    DatasetAPIException
        If the request to extend the dataset fails.
    errors.UnexpectedStatus
        If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
    httpx.TimeoutException
        If the request takes longer than Client.timeout.

    Examples
    --------
    >>> extended_dataset = extend_dataset(
    ...     prompt_settings={'model_alias': 'GPT-4o mini'},
    ...     prompt='Financial planning assistant that helps clients design an investment strategy.',
    ...     instructions='You are a financial planning assistant that helps clients design an investment strategy.',
    ...     examples=['I want to invest $1000 per month.'],
    ...     data_types=['Prompt Injection'],
    ...     count=3
    ... )
    >>> print('Extended dataset:', extended_dataset)
    """
    return Datasets().extend(
        prompt_settings=prompt_settings,
        prompt=prompt,
        instructions=instructions,
        examples=examples,
        data_types=data_types,
        count=count,
    )


def convert_dataset_row_to_record(dataset_row: DatasetRow) -> "DatasetRecord":
    values_dict = dataset_row.values_dict.to_dict()

    if "input" not in values_dict or not values_dict["input"]:
        raise ValueError("Dataset row must have input field")

    return DatasetRecord(
        id=dataset_row.row_id,
        input=values_dict["input"],
        output=values_dict.get("output", None),
        metadata=values_dict.get("metadata", None),
    )



import datetime as dt
import enum
import json
import logging
from asyncio import Queue
from collections.abc import Sequence
from dataclasses import is_dataclass
from datetime import date, datetime
from json import JSONEncoder
from pathlib import Path
from typing import Any
from uuid import UUID

from pydantic import BaseModel

from galileo.utils.dependencies import is_langchain_available

_logger = logging.getLogger(__name__)


def serialize_datetime(v: dt.datetime) -> str:
    """
    Serialize a datetime including timezone info.

    Uses the timezone info provided if present, otherwise uses the current runtime's timezone info.

    UTC datetimes end in "Z" while all other timezones are represented as offset from UTC, e.g. +05:00.
    """

    def _serialize_zoned_datetime(v: dt.datetime) -> str:
        if v.tzinfo is not None and v.tzinfo.tzname(None) == dt.timezone.utc.tzname(None):
            # UTC is a special case where we use "Z" at the end instead of "+00:00"
            return v.isoformat().replace("+00:00", "Z")
        # Delegate to the typical +/- offset format
        return v.isoformat()

    if v.tzinfo is not None:
        return _serialize_zoned_datetime(v)
    local_tz = dt.datetime.now().astimezone().tzinfo
    localized_dt = v.replace(tzinfo=local_tz)
    return _serialize_zoned_datetime(localized_dt)


def map_langchain_role(role: str) -> str:
    role_map = {"ai": "assistant", "human": "user"}
    return role_map.get(role, role)


class EventSerializer(JSONEncoder):
    """
    Custom JSON encoder to assist in the serialization of a wide range of objects.
    """

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        super().__init__(*args, **kwargs)
        self.seen: set[int] = set()  # Track seen objects to detect circular references

    def default(self, obj: Any) -> Any:
        try:
            if isinstance(obj, datetime):
                return serialize_datetime(obj)

            if isinstance(obj, (Exception, KeyboardInterrupt)):
                return f"{type(obj).__name__}: {obj!s}"

            if isinstance(obj, enum.Enum):
                return obj.value

            if isinstance(obj, Queue):
                return type(obj).__name__

            if isinstance(obj, UUID):
                return str(obj)

            if isinstance(obj, bytes):
                try:
                    return obj.decode("utf-8")
                except UnicodeDecodeError:
                    return "<not serializable bytes>"

            if isinstance(obj, date):
                return obj.isoformat()

            if isinstance(obj, Path):
                return str(obj)

            if is_langchain_available:
                from langchain_core.agents import AgentAction
                from langchain_core.documents import Document as LangchainDocument
                from langchain_core.load.serializable import Serializable
                from langchain_core.messages import AIMessage, AIMessageChunk, BaseMessage, ToolMessage
                from langchain_core.outputs import ChatGeneration, LLMResult
                from langchain_core.prompt_values import ChatPromptValue

                if isinstance(obj, (AgentAction, ChatPromptValue)):
                    return self.default(obj.messages)
                if isinstance(obj, ChatGeneration):
                    return self.default(obj.message)
                if isinstance(obj, LLMResult):
                    return self.default(obj.generations[0])
                if isinstance(obj, (AIMessageChunk, AIMessage)):
                    # Map the `type` to `role`.
                    dumped = obj.model_dump(mode="json", include={"content", "type", "additional_kwargs"})
                    content = dumped.get("content")
                    if isinstance(content, list):
                        # Responses API returns content as a list of dicts
                        # Convert list content to string format for consistency
                        if content and isinstance(content[0], dict):
                            dumped["content"] = content[0].get("text", "")
                    dumped["role"] = map_langchain_role(dumped.pop("type"))
                    additional_kwargs = dumped.pop("additional_kwargs", {})
                    if "tool_calls" in additional_kwargs:
                        dumped["tool_calls"] = additional_kwargs.pop("tool_calls")
                    if "reasoning" in additional_kwargs:
                        dumped["reasoning"] = additional_kwargs.pop("reasoning")
                    return dumped
                if isinstance(obj, ToolMessage):
                    # Map the `type` to `role`.
                    dumped = obj.model_dump(mode="json", include={"content", "type", "status", "tool_call_id"})
                    dumped["role"] = map_langchain_role(dumped.pop("type"))
                    return dumped
                if isinstance(obj, BaseMessage):
                    # Map the `type` to `role`.
                    dumped = obj.model_dump(mode="json", include={"content", "type"})
                    dumped["role"] = map_langchain_role(dumped.pop("type"))
                    return dumped

                if isinstance(obj, LangchainDocument):
                    return self.default(obj.model_dump(mode="json", include={"page_content", "metadata"}))

                if isinstance(obj, Serializable):
                    serialized = obj.to_json()
                    if "kwargs" in serialized:
                        kwargs = serialized["kwargs"]
                        kwargs.pop("type", None)
                        return kwargs
                    return serialized

            if is_dataclass(obj):
                return {self.default(k): self.default(v) for k, v in obj.__dict__.items()}

            # Handle Pydantic model classes (not instances)
            if isinstance(obj, type) and issubclass(obj, BaseModel):
                if hasattr(obj, "model_json_schema") and callable(obj.model_json_schema):
                    try:
                        return obj.model_json_schema()
                    except Exception:
                        # If schema generation fails, return class name
                        return f"<{obj.__name__}>"
                return f"<{obj.__name__}>"

            if isinstance(obj, BaseModel):
                return self.default(
                    obj.model_dump(mode="json", exclude_none=True, exclude_unset=True, exclude_defaults=True)
                )

            # 64-bit integers might overflow the JavaScript safe integer range.
            if isinstance(obj, int):
                return obj if self.is_js_safe_integer(obj) else str(obj)

            # Standard JSON-encodable types
            if isinstance(obj, (str, float, type(None))):
                return obj

            if isinstance(obj, (tuple, set, frozenset)):
                return list(obj)

            if isinstance(obj, dict):
                return {self.default(k): self.default(v) for k, v in obj.items()}

            if isinstance(obj, list):
                return [self.default(item) for item in obj]

            # Important: this needs to be always checked after str and bytes types
            # Useful for serializing protobuf messages
            if isinstance(obj, Sequence):
                return [self.default(item) for item in obj]

            if hasattr(obj, "__slots__") and len(obj.__slots__) > 0:
                return self.default({slot: getattr(obj, slot, None) for slot in obj.__slots__})

            if hasattr(obj, "__dict__"):
                obj_id = id(obj)

                if obj_id in self.seen:
                    # Break on circular references
                    return type(obj).__name__
                self.seen.add(obj_id)
                result = {k: self.default(v) for k, v in vars(obj).items() if not k.startswith("_")}
                self.seen.remove(obj_id)

                return result

            # Return object type rather than JSONEncoder.default(obj) which simply raises a TypeError
            return f"<{type(obj).__name__}>"

        except Exception:
            _logger.warning(f"Serialization failed for object of type {type(obj).__name__}")
            return f'"<not serializable object of type: {type(obj).__name__}>"'

    def encode(self, obj: Any) -> str:
        self.seen.clear()  # Clear seen objects before each encode call

        try:
            return super().encode(self.default(obj))
        except Exception:
            return f'"<not serializable object of type: {type(obj).__name__}>"'  # escaping the string to avoid JSON parsing errors

    @staticmethod
    def is_js_safe_integer(value: int) -> bool:
        """Ensure the value is within JavaScript's safe range for integers.

        Python's 64-bit integers can exceed this range, necessitating this check.
        https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Number/MAX_SAFE_INTEGER
        """
        max_safe_int = 2**53 - 1
        min_safe_int = -(2**53) + 1

        return min_safe_int <= value <= max_safe_int


def serialize_to_str(input_data: Any) -> str:
    """Safely serialize data to a JSON string."""
    if isinstance(input_data, str):
        return input_data

    if input_data is None or isinstance(input_data, (bool, int, float)):
        return json.dumps(input_data)

    try:
        # Use the EventSerializer for initial serialization
        serializer = EventSerializer()

        # First try to serialize directly using the serializer's default method
        processed_data = serializer.default(input_data)

        # Now encode it to a JSON string (this will always return a string)
        return serializer.encode(processed_data)
    except Exception:
        # Fallback if anything goes wrong
        _logger.warning(f"Serialization failed for object of type {type(input_data).__name__}")
        return ""


def convert_to_string_dict(input_: dict) -> dict[str, str]:
    """
    Convert a dict with arbitrary values to a dict[str, str] by converting
    all values to their string representations.
    """
    result: dict[str, str] = {}
    if not input_:
        return result
    for key, value in input_.items():
        # Ensure key is a string
        string_key = str(key)

        # Convert value to string based on type
        if value is None:
            string_value = ""
        elif isinstance(value, (dict, list, tuple)):
            # For complex types, use JSON serialization
            string_value = json.dumps(value, cls=EventSerializer)
        else:
            # For primitive types, convert directly to string
            string_value = str(value)

        result[string_key] = string_value

    return result


def convert_time_delta_to_ns(time_delta: dt.timedelta) -> int:
    """
    Convert a timedelta object to nanoseconds.
    """
    return int(time_delta.total_seconds() * 1e9)



from typing import Callable, Generic, Optional, TypeVar, Union

from pydantic import BaseModel, Field, ValidationError, field_validator, model_validator
from pydantic_core.core_schema import ValidationInfo

from galileo_core.schemas.logging.span import Span
from galileo_core.schemas.logging.step import STEP_TYPES_WITH_CHILD_SPANS, StepType
from galileo_core.schemas.logging.trace import Trace
from galileo_core.schemas.shared.metric import MetricValueType
from galileo_core.schemas.shared.scorers.scorer_name import ScorerName as GalileoScorers

MetricType = TypeVar("MetricType", bound=MetricValueType)


class LocalMetricConfig(BaseModel, Generic[MetricType]):
    name: str = Field(description="Name of the local metric")
    scorer_fn: Callable[[Union[Trace, Span]], MetricType] = Field(
        description="function to call to produce the metric value (takes a trace or span as input"
    )
    aggregator_fn: Optional[Callable[[list[MetricType]], Union[MetricType, dict[str, MetricType]]]] = Field(
        default=None,
        description="function to call to produce the aggregate metric values from individual metric values",
    )
    scorable_types: list[StepType] = Field(default=[StepType.llm])
    aggregatable_types: list[StepType] = Field(default=[StepType.trace])

    @field_validator("aggregatable_types", mode="before")
    def set_aggregatable_types(cls, value: list[StepType], info: ValidationInfo) -> list[StepType]:
        if "scorable_types" in info.data and any(
            scorable_type in value for scorable_type in info.data["scorable_types"]
        ):
            raise ValidationError("aggregatable_types cannot contain any types in scorable_types")
        for step_type in value:
            if step_type not in STEP_TYPES_WITH_CHILD_SPANS:
                raise ValidationError("aggregatable_types can only contain trace or workflow steps")
        return value


class Metric(BaseModel):
    name: str = Field(
        description="The name of the metric you want to run a specific version of (ie: 'Sentence Density')."
    )
    version: Optional[int] = Field(
        default=None,
        description="The version of the metric (ie: 1, 2, 3, etc.). If None is provided, the 'default' version will be used.",
    )

    @model_validator(mode="after")
    def validate_name_and_version(self) -> "Metric":
        preset_metric_names = [scorer.value for scorer in GalileoScorers]
        if self.name in preset_metric_names:
            if self.version is not None:
                raise ValueError(
                    f"Galileo metric's '{self.name}' do not support versioning at this time. Please use the default version."
                )
        return self



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.experiment_dataset_request import ExperimentDatasetRequest
    from ..models.prompt_run_settings import PromptRunSettings
    from ..models.scorer_config import ScorerConfig


T = TypeVar("T", bound="ExperimentCreateRequest")


@_attrs_define
class ExperimentCreateRequest:
    """
    Attributes:
        name (str):
        dataset (Union['ExperimentDatasetRequest', None, Unset]):
        playground_id (Union[None, Unset, str]):
        playground_prompt_id (Union[None, Unset, str]):
        prompt_settings (Union[Unset, PromptRunSettings]): Prompt run settings.
        prompt_template_version_id (Union[None, Unset, str]):
        scorers (Union[Unset, list['ScorerConfig']]):
        task_type (Union[Literal[16], Literal[17], Unset]):  Default: 16.
        trigger (Union[Unset, bool]):  Default: False.
    """

    name: str
    dataset: Union["ExperimentDatasetRequest", None, Unset] = UNSET
    playground_id: Union[None, Unset, str] = UNSET
    playground_prompt_id: Union[None, Unset, str] = UNSET
    prompt_settings: Union[Unset, "PromptRunSettings"] = UNSET
    prompt_template_version_id: Union[None, Unset, str] = UNSET
    scorers: Union[Unset, list["ScorerConfig"]] = UNSET
    task_type: Union[Literal[16], Literal[17], Unset] = 16
    trigger: Union[Unset, bool] = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.experiment_dataset_request import ExperimentDatasetRequest

        name = self.name

        dataset: Union[None, Unset, dict[str, Any]]
        if isinstance(self.dataset, Unset):
            dataset = UNSET
        elif isinstance(self.dataset, ExperimentDatasetRequest):
            dataset = self.dataset.to_dict()
        else:
            dataset = self.dataset

        playground_id: Union[None, Unset, str]
        playground_id = UNSET if isinstance(self.playground_id, Unset) else self.playground_id

        playground_prompt_id: Union[None, Unset, str]
        playground_prompt_id = UNSET if isinstance(self.playground_prompt_id, Unset) else self.playground_prompt_id

        prompt_settings: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.prompt_settings, Unset):
            prompt_settings = self.prompt_settings.to_dict()

        prompt_template_version_id: Union[None, Unset, str]
        if isinstance(self.prompt_template_version_id, Unset):
            prompt_template_version_id = UNSET
        else:
            prompt_template_version_id = self.prompt_template_version_id

        scorers: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.scorers, Unset):
            scorers = []
            for scorers_item_data in self.scorers:
                scorers_item = scorers_item_data.to_dict()
                scorers.append(scorers_item)

        task_type: Union[Literal[16], Literal[17], Unset]
        task_type = UNSET if isinstance(self.task_type, Unset) else self.task_type

        trigger = self.trigger

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"name": name})
        if dataset is not UNSET:
            field_dict["dataset"] = dataset
        if playground_id is not UNSET:
            field_dict["playground_id"] = playground_id
        if playground_prompt_id is not UNSET:
            field_dict["playground_prompt_id"] = playground_prompt_id
        if prompt_settings is not UNSET:
            field_dict["prompt_settings"] = prompt_settings
        if prompt_template_version_id is not UNSET:
            field_dict["prompt_template_version_id"] = prompt_template_version_id
        if scorers is not UNSET:
            field_dict["scorers"] = scorers
        if task_type is not UNSET:
            field_dict["task_type"] = task_type
        if trigger is not UNSET:
            field_dict["trigger"] = trigger

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.experiment_dataset_request import ExperimentDatasetRequest
        from ..models.prompt_run_settings import PromptRunSettings
        from ..models.scorer_config import ScorerConfig

        d = dict(src_dict)
        name = d.pop("name")

        def _parse_dataset(data: object) -> Union["ExperimentDatasetRequest", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return ExperimentDatasetRequest.from_dict(data)

            except:  # noqa: E722
                pass
            return cast(Union["ExperimentDatasetRequest", None, Unset], data)

        dataset = _parse_dataset(d.pop("dataset", UNSET))

        def _parse_playground_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        playground_id = _parse_playground_id(d.pop("playground_id", UNSET))

        def _parse_playground_prompt_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        playground_prompt_id = _parse_playground_prompt_id(d.pop("playground_prompt_id", UNSET))

        _prompt_settings = d.pop("prompt_settings", UNSET)
        prompt_settings: Union[Unset, PromptRunSettings]
        if isinstance(_prompt_settings, Unset):
            prompt_settings = UNSET
        else:
            prompt_settings = PromptRunSettings.from_dict(_prompt_settings)

        def _parse_prompt_template_version_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        prompt_template_version_id = _parse_prompt_template_version_id(d.pop("prompt_template_version_id", UNSET))

        scorers = []
        _scorers = d.pop("scorers", UNSET)
        for scorers_item_data in _scorers or []:
            scorers_item = ScorerConfig.from_dict(scorers_item_data)

            scorers.append(scorers_item)

        def _parse_task_type(data: object) -> Union[Literal[16], Literal[17], Unset]:
            if isinstance(data, Unset):
                return data
            task_type_type_0 = cast(Literal[16], data)
            if task_type_type_0 != 16:
                raise ValueError(f"task_type_type_0 must match const 16, got '{task_type_type_0}'")
            return task_type_type_0
            task_type_type_1 = cast(Literal[17], data)
            if task_type_type_1 != 17:
                raise ValueError(f"task_type_type_1 must match const 17, got '{task_type_type_1}'")
            return task_type_type_1

        task_type = _parse_task_type(d.pop("task_type", UNSET))

        trigger = d.pop("trigger", UNSET)

        experiment_create_request = cls(
            name=name,
            dataset=dataset,
            playground_id=playground_id,
            playground_prompt_id=playground_prompt_id,
            prompt_settings=prompt_settings,
            prompt_template_version_id=prompt_template_version_id,
            scorers=scorers,
            task_type=task_type,
            trigger=trigger,
        )

        experiment_create_request.additional_properties = d
        return experiment_create_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.dataset_row_metadata import DatasetRowMetadata
    from ..models.dataset_row_values_dict import DatasetRowValuesDict
    from ..models.dataset_row_values_item_type_3 import DatasetRowValuesItemType3


T = TypeVar("T", bound="DatasetRow")


@_attrs_define
class DatasetRow:
    """
    Attributes:
        index (int):
        metadata (Union['DatasetRowMetadata', None]):
        row_id (str):
        values (list[Union['DatasetRowValuesItemType3', None, float, int, str]]):
        values_dict (DatasetRowValuesDict):
    """

    index: int
    metadata: Union["DatasetRowMetadata", None]
    row_id: str
    values: list[Union["DatasetRowValuesItemType3", None, float, int, str]]
    values_dict: "DatasetRowValuesDict"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.dataset_row_metadata import DatasetRowMetadata
        from ..models.dataset_row_values_item_type_3 import DatasetRowValuesItemType3

        index = self.index

        metadata: Union[None, dict[str, Any]]
        metadata = self.metadata.to_dict() if isinstance(self.metadata, DatasetRowMetadata) else self.metadata

        row_id = self.row_id

        values = []
        for values_item_data in self.values:
            values_item: Union[None, dict[str, Any], float, int, str]
            if isinstance(values_item_data, DatasetRowValuesItemType3):
                values_item = values_item_data.to_dict()
            else:
                values_item = values_item_data
            values.append(values_item)

        values_dict = self.values_dict.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {"index": index, "metadata": metadata, "row_id": row_id, "values": values, "values_dict": values_dict}
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.dataset_row_metadata import DatasetRowMetadata
        from ..models.dataset_row_values_dict import DatasetRowValuesDict
        from ..models.dataset_row_values_item_type_3 import DatasetRowValuesItemType3

        d = dict(src_dict)
        index = d.pop("index")

        def _parse_metadata(data: object) -> Union["DatasetRowMetadata", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return DatasetRowMetadata.from_dict(data)

            except:  # noqa: E722
                pass
            return cast(Union["DatasetRowMetadata", None], data)

        metadata = _parse_metadata(d.pop("metadata"))

        row_id = d.pop("row_id")

        values = []
        _values = d.pop("values")
        for values_item_data in _values:

            def _parse_values_item(data: object) -> Union["DatasetRowValuesItemType3", None, float, int, str]:
                if data is None:
                    return data
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    return DatasetRowValuesItemType3.from_dict(data)

                except:  # noqa: E722
                    pass
                return cast(Union["DatasetRowValuesItemType3", None, float, int, str], data)

            values_item = _parse_values_item(values_item_data)

            values.append(values_item)

        values_dict = DatasetRowValuesDict.from_dict(d.pop("values_dict"))

        dataset_row = cls(index=index, metadata=metadata, row_id=row_id, values=values, values_dict=values_dict)

        dataset_row.additional_properties = d
        return dataset_row

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.project_labels import ProjectLabels
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.permission import Permission
    from ..models.user_info import UserInfo


T = TypeVar("T", bound="ProjectItem")


@_attrs_define
class ProjectItem:
    """Represents a single project item for the UI list.

    Attributes:
        created_at (datetime.datetime):
        id (str):
        name (str):
        updated_at (datetime.datetime):
        bookmark (Union[Unset, bool]):  Default: False.
        created_by_user (Union['UserInfo', None, Unset]):
        description (Union[None, Unset, str]):
        labels (Union[Unset, list[ProjectLabels]]): List of labels associated with the project.
        num_experiments (Union[None, Unset, int]): Count of runs with task_type=16
        num_logstreams (Union[None, Unset, int]): Count of runs with task_type=15
        permissions (Union[Unset, list['Permission']]):
    """

    created_at: datetime.datetime
    id: str
    name: str
    updated_at: datetime.datetime
    bookmark: Union[Unset, bool] = False
    created_by_user: Union["UserInfo", None, Unset] = UNSET
    description: Union[None, Unset, str] = UNSET
    labels: Union[Unset, list[ProjectLabels]] = UNSET
    num_experiments: Union[None, Unset, int] = UNSET
    num_logstreams: Union[None, Unset, int] = UNSET
    permissions: Union[Unset, list["Permission"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.user_info import UserInfo

        created_at = self.created_at.isoformat()

        id = self.id

        name = self.name

        updated_at = self.updated_at.isoformat()

        bookmark = self.bookmark

        created_by_user: Union[None, Unset, dict[str, Any]]
        if isinstance(self.created_by_user, Unset):
            created_by_user = UNSET
        elif isinstance(self.created_by_user, UserInfo):
            created_by_user = self.created_by_user.to_dict()
        else:
            created_by_user = self.created_by_user

        description: Union[None, Unset, str]
        description = UNSET if isinstance(self.description, Unset) else self.description

        labels: Union[Unset, list[str]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = []
            for labels_item_data in self.labels:
                labels_item = labels_item_data.value
                labels.append(labels_item)

        num_experiments: Union[None, Unset, int]
        num_experiments = UNSET if isinstance(self.num_experiments, Unset) else self.num_experiments

        num_logstreams: Union[None, Unset, int]
        num_logstreams = UNSET if isinstance(self.num_logstreams, Unset) else self.num_logstreams

        permissions: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = []
            for permissions_item_data in self.permissions:
                permissions_item = permissions_item_data.to_dict()
                permissions.append(permissions_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"created_at": created_at, "id": id, "name": name, "updated_at": updated_at})
        if bookmark is not UNSET:
            field_dict["bookmark"] = bookmark
        if created_by_user is not UNSET:
            field_dict["created_by_user"] = created_by_user
        if description is not UNSET:
            field_dict["description"] = description
        if labels is not UNSET:
            field_dict["labels"] = labels
        if num_experiments is not UNSET:
            field_dict["num_experiments"] = num_experiments
        if num_logstreams is not UNSET:
            field_dict["num_logstreams"] = num_logstreams
        if permissions is not UNSET:
            field_dict["permissions"] = permissions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.permission import Permission
        from ..models.user_info import UserInfo

        d = dict(src_dict)
        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id")

        name = d.pop("name")

        updated_at = isoparse(d.pop("updated_at"))

        bookmark = d.pop("bookmark", UNSET)

        def _parse_created_by_user(data: object) -> Union["UserInfo", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                return UserInfo.from_dict(data)

            except:  # noqa: E722
                pass
            return cast(Union["UserInfo", None, Unset], data)

        created_by_user = _parse_created_by_user(d.pop("created_by_user", UNSET))

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        labels = []
        _labels = d.pop("labels", UNSET)
        for labels_item_data in _labels or []:
            labels_item = ProjectLabels(labels_item_data)

            labels.append(labels_item)

        def _parse_num_experiments(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        num_experiments = _parse_num_experiments(d.pop("num_experiments", UNSET))

        def _parse_num_logstreams(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        num_logstreams = _parse_num_logstreams(d.pop("num_logstreams", UNSET))

        permissions = []
        _permissions = d.pop("permissions", UNSET)
        for permissions_item_data in _permissions or []:
            permissions_item = Permission.from_dict(permissions_item_data)

            permissions.append(permissions_item)

        project_item = cls(
            created_at=created_at,
            id=id,
            name=name,
            updated_at=updated_at,
            bookmark=bookmark,
            created_by_user=created_by_user,
            description=description,
            labels=labels,
            num_experiments=num_experiments,
            num_logstreams=num_logstreams,
            permissions=permissions,
        )

        project_item.additional_properties = d
        return project_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.few_shot_example import FewShotExample


T = TypeVar("T", bound="GroundednessTemplate")


@_attrs_define
class GroundednessTemplate:
    r"""Template for the groundedness metric,
    containing all the info necessary to send the groundedness prompt.

        Attributes:
            explanation_field_name (Union[Unset, str]): Field name to look for in the chainpoll response, for the
                explanation. Default: 'explanation'.
            metric_description (Union[Unset, str]):  Default: 'I have a RAG (retrieval-augmented generation) system that
                generates text based on one or more documents that I always include in my prompts. I want a metric that checks
                whether the generated text was supported by information in the documents. The metric should exhaustively check
                each claim in the response against the documents, one by one, listing them out explicitly.'.
            metric_few_shot_examples (Union[Unset, list['FewShotExample']]):
            metric_system_prompt (Union[Unset, str]):  Default: 'The user will provide you with a prompt that was sent to an
                automatic question-answering system, and that system\'s response. Both will be provided as JSON strings.\n\nThe
                prompt will contain one or more documents intended as context which the question-answering system was given as
                reference material.\n\nYour task is to determine whether the answer was supported by the documents.\n\nThink
                step by step, and explain your reasoning carefully.\nState your observations first, before drawing any
                conclusions.\n\nRespond in the following JSON format:\n\n```\n{\n    \\"explanation\\": string,\n
                \\"was_supported\\": boolean\n}\n```\n\n\\"explanation\\": Your step-by-step reasoning process. List out the
                claims made in the response, and for each claim, provide a detailed explanation of why that claim is or is not
                supported by the documents.\n\n\\"was_supported\\": `true` if the response was supported by the documents,
                `false` otherwise.\n\nYou must respond with valid JSON.'.
            template (Union[Unset, str]):  Default: 'Prompt JSON:\n\n```\n{query_json}\n```\n\nResponse
                JSON:\n\n```\n{response_json}\n```'.
            value_field_name (Union[Unset, str]):  Default: 'was_supported'.
    """

    explanation_field_name: Union[Unset, str] = "explanation"
    metric_description: Union[Unset, str] = (
        "I have a RAG (retrieval-augmented generation) system that generates text based on one or more documents that I always include in my prompts. I want a metric that checks whether the generated text was supported by information in the documents. The metric should exhaustively check each claim in the response against the documents, one by one, listing them out explicitly."
    )
    metric_few_shot_examples: Union[Unset, list["FewShotExample"]] = UNSET
    metric_system_prompt: Union[Unset, str] = (
        'The user will provide you with a prompt that was sent to an automatic question-answering system, and that system\'s response. Both will be provided as JSON strings.\n\nThe prompt will contain one or more documents intended as context which the question-answering system was given as reference material.\n\nYour task is to determine whether the answer was supported by the documents.\n\nThink step by step, and explain your reasoning carefully.\nState your observations first, before drawing any conclusions.\n\nRespond in the following JSON format:\n\n```\n{\n    \\"explanation\\": string,\n    \\"was_supported\\": boolean\n}\n```\n\n\\"explanation\\": Your step-by-step reasoning process. List out the claims made in the response, and for each claim, provide a detailed explanation of why that claim is or is not supported by the documents.\n\n\\"was_supported\\": `true` if the response was supported by the documents, `false` otherwise.\n\nYou must respond with valid JSON.'
    )
    template: Union[Unset, str] = (
        "Prompt JSON:\n\n```\n{query_json}\n```\n\nResponse JSON:\n\n```\n{response_json}\n```"
    )
    value_field_name: Union[Unset, str] = "was_supported"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        explanation_field_name = self.explanation_field_name

        metric_description = self.metric_description

        metric_few_shot_examples: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.metric_few_shot_examples, Unset):
            metric_few_shot_examples = []
            for metric_few_shot_examples_item_data in self.metric_few_shot_examples:
                metric_few_shot_examples_item = metric_few_shot_examples_item_data.to_dict()
                metric_few_shot_examples.append(metric_few_shot_examples_item)

        metric_system_prompt = self.metric_system_prompt

        template = self.template

        value_field_name = self.value_field_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if explanation_field_name is not UNSET:
            field_dict["explanation_field_name"] = explanation_field_name
        if metric_description is not UNSET:
            field_dict["metric_description"] = metric_description
        if metric_few_shot_examples is not UNSET:
            field_dict["metric_few_shot_examples"] = metric_few_shot_examples
        if metric_system_prompt is not UNSET:
            field_dict["metric_system_prompt"] = metric_system_prompt
        if template is not UNSET:
            field_dict["template"] = template
        if value_field_name is not UNSET:
            field_dict["value_field_name"] = value_field_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.few_shot_example import FewShotExample

        d = dict(src_dict)
        explanation_field_name = d.pop("explanation_field_name", UNSET)

        metric_description = d.pop("metric_description", UNSET)

        metric_few_shot_examples = []
        _metric_few_shot_examples = d.pop("metric_few_shot_examples", UNSET)
        for metric_few_shot_examples_item_data in _metric_few_shot_examples or []:
            metric_few_shot_examples_item = FewShotExample.from_dict(metric_few_shot_examples_item_data)

            metric_few_shot_examples.append(metric_few_shot_examples_item)

        metric_system_prompt = d.pop("metric_system_prompt", UNSET)

        template = d.pop("template", UNSET)

        value_field_name = d.pop("value_field_name", UNSET)

        groundedness_template = cls(
            explanation_field_name=explanation_field_name,
            metric_description=metric_description,
            metric_few_shot_examples=metric_few_shot_examples,
            metric_system_prompt=metric_system_prompt,
            template=template,
            value_field_name=value_field_name,
        )

        groundedness_template.additional_properties = d
        return groundedness_template

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AgentType(str, Enum):
    CLASSIFIER = "classifier"
    DEFAULT = "default"
    JUDGE = "judge"
    PLANNER = "planner"
    REACT = "react"
    REFLECTION = "reflection"
    ROUTER = "router"
    SUPERVISOR = "supervisor"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="HallucinationSegment")


@_attrs_define
class HallucinationSegment:
    """
    Attributes:
        end (int):
        hallucination (float):
        start (int):
        hallucination_severity (Union[Unset, int]):  Default: 0.
    """

    end: int
    hallucination: float
    start: int
    hallucination_severity: Union[Unset, int] = 0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        end = self.end

        hallucination = self.hallucination

        start = self.start

        hallucination_severity = self.hallucination_severity

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"end": end, "hallucination": hallucination, "start": start})
        if hallucination_severity is not UNSET:
            field_dict["hallucination_severity"] = hallucination_severity

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        end = d.pop("end")

        hallucination = d.pop("hallucination")

        start = d.pop("start")

        hallucination_severity = d.pop("hallucination_severity", UNSET)

        hallucination_segment = cls(
            end=end, hallucination=hallucination, start=start, hallucination_severity=hallucination_severity
        )

        hallucination_segment.additional_properties = d
        return hallucination_segment

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Experiment Tags functionality for managing tags on experiments."""

import logging
from typing import Union

from galileo.config import GalileoPythonConfig
from galileo.resources.api.experiment_tags import (
    delete_experiment_tag_projects_project_id_experiments_experiment_id_tags_tag_id_delete,
    get_experiment_tags_projects_project_id_experiments_experiment_id_tags_get,
    set_tag_for_experiment_projects_project_id_experiments_experiment_id_tags_post,
)
from galileo.resources.models.http_validation_error import HTTPValidationError
from galileo.resources.models.run_tag_create_request import RunTagCreateRequest
from galileo.resources.models.run_tag_db import RunTagDB
from galileo.utils.catch_log import DecorateAllMethods
from galileo.utils.exceptions import APIException

_logger = logging.getLogger(__name__)


class ExperimentTagsAPIException(APIException):
    """Exception raised when experiment tags operations fail."""


class ExperimentTag(RunTagDB):
    """Wrapper class for experiment tags that provides additional functionality."""

    def __init__(self, experiment_tag: Union[None, RunTagDB] = None):
        """
        Initialize an ExperimentTag instance.

        Parameters
        ----------
        experiment_tag : Union[None, RunTagDB], optional
            The experiment tag data to initialize from. If None, creates an empty instance.
            Defaults to None.
        """
        if experiment_tag is not None:
            super().__init__(
                id=experiment_tag.id,
                project_id=experiment_tag.project_id,
                run_id=experiment_tag.run_id,
                key=experiment_tag.key,
                value=experiment_tag.value,
                tag_type=experiment_tag.tag_type,
                created_at=experiment_tag.created_at,
                updated_at=experiment_tag.updated_at,
                created_by=experiment_tag.created_by,
            )
            self.additional_properties = experiment_tag.additional_properties.copy()


class ExperimentTags(DecorateAllMethods):
    config: GalileoPythonConfig

    def __init__(self) -> None:
        self.config = GalileoPythonConfig.get()

    def get_experiment_tags(self, project_id: str, experiment_id: str) -> list[ExperimentTag]:
        """
        Get all tags for a specific experiment.

        Args:
            project_id: The project ID
            experiment_id: The experiment ID

        Returns:
            list[ExperimentTag]: List of tags associated with the experiment

        Raises:
            ExperimentTagsAPIException: If the API call fails
            ValueError: If the experiment is not found
        """
        response = get_experiment_tags_projects_project_id_experiments_experiment_id_tags_get.sync(
            project_id=project_id, experiment_id=experiment_id, client=self.config.api_client
        )

        if isinstance(response, HTTPValidationError):
            raise ExperimentTagsAPIException(f"Failed to get experiment tags: {response.detail}")

        return response

    def upsert_experiment_tag(
        self, project_id: str, experiment_id: str, key: str, value: str, tag_type: str = "generic"
    ) -> ExperimentTag:
        """
        Upsert a tag for a specific experiment.

        Args:
            project_id: The project ID
            experiment_id: The experiment ID
            key: The tag key
            value: The tag value
            tag_type: The type of tag (default: "generic")

        Returns:
            ExperimentTag: The created or updated tag

        Raises:
            ExperimentTagsAPIException: If the API call fails
            ValueError: If the experiment is not found
        """
        request_body = RunTagCreateRequest(key=key, value=value, tag_type=tag_type)

        response = set_tag_for_experiment_projects_project_id_experiments_experiment_id_tags_post.sync(
            project_id=project_id, experiment_id=experiment_id, client=self.config.api_client, body=request_body
        )

        if isinstance(response, HTTPValidationError):
            raise ExperimentTagsAPIException(f"Failed to upsert experiment tag: {response.detail}")

        if not response:
            raise ExperimentTagsAPIException("No response received from API")

        return ExperimentTag(experiment_tag=response)

    def delete_experiment_tag(self, project_id: str, experiment_id: str, tag_id: str) -> dict[str, str]:
        """
        Delete a specific tag from an experiment.

        Args:
            project_id: The project ID
            experiment_id: The experiment ID
            tag_id: The tag ID to delete

        Returns:
            dict[str, str]: Success message

        Raises:
            ExperimentTagsAPIException: If the API call fails
            ValueError: If the experiment or tag is not found
        """
        response = delete_experiment_tag_projects_project_id_experiments_experiment_id_tags_tag_id_delete.sync(
            project_id=project_id, experiment_id=experiment_id, tag_id=tag_id, client=self.config.api_client
        )

        if isinstance(response, HTTPValidationError):
            raise ExperimentTagsAPIException(f"Failed to delete experiment tag: {response.detail}")

        if not response:
            raise ExperimentTagsAPIException("No response received from API")

        return {
            "message": response.message if hasattr(response, "message") else "Tag deleted successfully",
            "tag_id": tag_id,
        }


def get_experiment_tags(project_id: str, experiment_id: str) -> list[ExperimentTag]:
    """
    Get all tags for a specific experiment.

    Args:
        project_id: The project ID
        experiment_id: The experiment ID

    Returns:
        list[ExperimentTag]: List of tags associated with the experiment

    Raises:
        ExperimentTagsAPIException: If the API call fails
        ValueError: If the experiment is not found
    """
    return ExperimentTags().get_experiment_tags(project_id, experiment_id)


def upsert_experiment_tag(
    project_id: str, experiment_id: str, key: str, value: str, tag_type: str = "generic"
) -> ExperimentTag:
    """
    Upsert (create or update) a tag for a specific experiment.

    Args:
        project_id: The project ID
        experiment_id: The experiment ID
        key: The tag key
        value: The tag value
        tag_type: The type of tag (default: "generic")

    Returns:
        ExperimentTag: The created or updated tag

    Raises:
        ExperimentTagsAPIException: If the API call fails
        ValueError: If the experiment is not found
    """
    return ExperimentTags().upsert_experiment_tag(project_id, experiment_id, key, value, tag_type)


def delete_experiment_tag(project_id: str, experiment_id: str, tag_id: str) -> dict[str, str]:
    """
    Delete a specific tag from an experiment.

    Args:
        project_id: The project ID
        experiment_id: The experiment ID
        tag_id: The tag ID to delete

    Returns:
        dict[str, str]: Success message

    Raises:
        ExperimentTagsAPIException: If the API call fails
        ValueError: If the experiment or tag is not found
    """
    return ExperimentTags().delete_experiment_tag(project_id, experiment_id, tag_id)



from enum import Enum


class FeedbackType(str, Enum):
    LIKE_DISLIKE = "like_dislike"
    SCORE = "score"
    STAR = "star"
    TAGS = "tags"
    TEXT = "text"

    def __str__(self) -> str:
        return str(self.value)



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.execution_status import ExecutionStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.trace_metadata import TraceMetadata


T = TypeVar("T", bound="ProtectResponse")


@_attrs_define
class ProtectResponse:
    """Protect response schema with custom OpenAPI title.

    Attributes:
        text (str): Text from the request after processing the rules.
        trace_metadata (TraceMetadata):
        status (Union[Unset, ExecutionStatus]): Status of the execution.
    """

    text: str
    trace_metadata: "TraceMetadata"
    status: Union[Unset, ExecutionStatus] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        text = self.text

        trace_metadata = self.trace_metadata.to_dict()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"text": text, "trace_metadata": trace_metadata})
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.trace_metadata import TraceMetadata

        d = dict(src_dict)
        text = d.pop("text")

        trace_metadata = TraceMetadata.from_dict(d.pop("trace_metadata"))

        _status = d.pop("status", UNSET)
        status: Union[Unset, ExecutionStatus]
        status = UNSET if isinstance(_status, Unset) else ExecutionStatus(_status)

        protect_response = cls(text=text, trace_metadata=trace_metadata, status=status)

        protect_response.additional_properties = d
        return protect_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.logging_method import LoggingMethod
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.trace import Trace


T = TypeVar("T", bound="LogTracesIngestRequest")


@_attrs_define
class LogTracesIngestRequest:
    """Request model for ingesting traces.

    Attributes:
        traces (list['Trace']): List of traces to log.
        client_version (Union[None, Unset, str]):
        experiment_id (Union[None, Unset, str]): Experiment id associated with the traces.
        is_complete (Union[Unset, bool]): Whether or not the records in this request are complete. Default: True.
        log_stream_id (Union[None, Unset, str]): Log stream id associated with the traces.
        logging_method (Union[Unset, LoggingMethod]):
        metrics_testing_id (Union[None, Unset, str]): Metrics testing id associated with the traces.
        reliable (Union[Unset, bool]): Whether or not to use reliable logging.  If set to False, the method will respond
            immediately before verifying that the traces have been successfully ingested, and no error message will be
            returned if ingestion fails.  If set to True, the method will wait for the traces to be successfully ingested or
            return an error message if there is an ingestion failure. Default: False.
        session_id (Union[None, Unset, str]): Session id associated with the traces.
    """

    traces: list["Trace"]
    client_version: Union[None, Unset, str] = UNSET
    experiment_id: Union[None, Unset, str] = UNSET
    is_complete: Union[Unset, bool] = True
    log_stream_id: Union[None, Unset, str] = UNSET
    logging_method: Union[Unset, LoggingMethod] = UNSET
    metrics_testing_id: Union[None, Unset, str] = UNSET
    reliable: Union[Unset, bool] = False
    session_id: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        traces = []
        for traces_item_data in self.traces:
            traces_item = traces_item_data.to_dict()
            traces.append(traces_item)

        client_version: Union[None, Unset, str]
        client_version = UNSET if isinstance(self.client_version, Unset) else self.client_version

        experiment_id: Union[None, Unset, str]
        experiment_id = UNSET if isinstance(self.experiment_id, Unset) else self.experiment_id

        is_complete = self.is_complete

        log_stream_id: Union[None, Unset, str]
        log_stream_id = UNSET if isinstance(self.log_stream_id, Unset) else self.log_stream_id

        logging_method: Union[Unset, str] = UNSET
        if not isinstance(self.logging_method, Unset):
            logging_method = self.logging_method.value

        metrics_testing_id: Union[None, Unset, str]
        metrics_testing_id = UNSET if isinstance(self.metrics_testing_id, Unset) else self.metrics_testing_id

        reliable = self.reliable

        session_id: Union[None, Unset, str]
        session_id = UNSET if isinstance(self.session_id, Unset) else self.session_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({"traces": traces})
        if client_version is not UNSET:
            field_dict["client_version"] = client_version
        if experiment_id is not UNSET:
            field_dict["experiment_id"] = experiment_id
        if is_complete is not UNSET:
            field_dict["is_complete"] = is_complete
        if log_stream_id is not UNSET:
            field_dict["log_stream_id"] = log_stream_id
        if logging_method is not UNSET:
            field_dict["logging_method"] = logging_method
        if metrics_testing_id is not UNSET:
            field_dict["metrics_testing_id"] = metrics_testing_id
        if reliable is not UNSET:
            field_dict["reliable"] = reliable
        if session_id is not UNSET:
            field_dict["session_id"] = session_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.trace import Trace

        d = dict(src_dict)
        traces = []
        _traces = d.pop("traces")
        for traces_item_data in _traces:
            traces_item = Trace.from_dict(traces_item_data)

            traces.append(traces_item)

        def _parse_client_version(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        client_version = _parse_client_version(d.pop("client_version", UNSET))

        def _parse_experiment_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        experiment_id = _parse_experiment_id(d.pop("experiment_id", UNSET))

        is_complete = d.pop("is_complete", UNSET)

        def _parse_log_stream_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        log_stream_id = _parse_log_stream_id(d.pop("log_stream_id", UNSET))

        _logging_method = d.pop("logging_method", UNSET)
        logging_method: Union[Unset, LoggingMethod]
        logging_method = UNSET if isinstance(_logging_method, Unset) else LoggingMethod(_logging_method)

        def _parse_metrics_testing_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        metrics_testing_id = _parse_metrics_testing_id(d.pop("metrics_testing_id", UNSET))

        reliable = d.pop("reliable", UNSET)

        def _parse_session_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        session_id = _parse_session_id(d.pop("session_id", UNSET))

        log_traces_ingest_request = cls(
            traces=traces,
            client_version=client_version,
            experiment_id=experiment_id,
            is_complete=is_complete,
            log_stream_id=log_stream_id,
            logging_method=logging_method,
            metrics_testing_id=metrics_testing_id,
            reliable=reliable,
            session_id=session_id,
        )

        log_traces_ingest_request.additional_properties = d
        return log_traces_ingest_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
